@@ -1,13 +1,51 @@
-use crate::common;
+use crate::attr;
 use proc_macro2::TokenStream;
 use quote::quote;
 use syn::{Data, DataEnum, DataStruct, DeriveInput, Error, Fields, FieldsNamed, Result};
 
 pub fn derive(node: &DeriveInput) -> Result<TokenStream> {
+    let attrs = attr::get(&node.attrs)?;
+    attrs.disallow_optional()?;
+    attrs.disallow_tag()?;
+    attrs.disallow_flatten()?;
     match &node.data {
-        Data::Struct(strut) => derive_struct(node, strut),
+        Data::Struct(strut) => match &strut.fields {
+            Fields::Named(fields) => {
+                if attrs.untagged.is_some() {
+                    derive_untagged_struct(node, strut, fields)
+                } else {
+                    derive_struct(node, strut, fields)
+                }
+            }
+            Fields::Unnamed(fields) => {
+                attrs.disallow_untagged()?;
+                let len = fields.unnamed.len();
+                match len {
+                    0 => {
+                        return Err(Error::new_spanned(
+                            node,
+                            "empty tuple structs as deserialize are not supported",
+                        ));
+                    }
+                    1 => derive_newtype_struct(node, strut, &fields.unnamed[0]),
+                    _ => {
+                        return Err(Error::new_spanned(
+                            node,
+                            "tuple structs as deserialize are not supported",
+                        ))
+                    }
+                }
+            }
+            Fields::Unit => {
+                attrs.disallow_untagged()?;
+                return Err(Error::new_spanned(
+                    node,
+                    "unit structs as deserialize are not supported",
+                ));
+            }
+        },
         Data::Enum(enu) => {
-            if common::has_untagged(&node.attrs) {
+            if attrs.untagged.is_some() {
                 derive_untagged_enum(node, enu)
             } else {
                 derive_enum(node, enu)
@@ -20,43 +58,7 @@ pub fn derive(node: &DeriveInput) -> Result<TokenStream> {
     }
 }
 
-fn derive_struct(node: &DeriveInput, strut: &DataStruct) -> Result<TokenStream> {
-    match &strut.fields {
-        Fields::Named(fields) => {
-            if common::has_untagged(&node.attrs) {
-                derive_untagged_c_struct(node, strut, fields)
-            } else {
-                derive_c_struct(node, strut, fields)
-            }
-        }
-        Fields::Unnamed(fields) => {
-            let len = fields.unnamed.len();
-            match len {
-                0 => {
-                    return Err(Error::new_spanned(
-                        node,
-                        "empty tuple structs as deserialize are not supported",
-                    ));
-                }
-                1 => derive_newtype_struct(node, strut, &fields.unnamed[0]),
-                _ => {
-                    return Err(Error::new_spanned(
-                        node,
-                        "tuple structs as deserialize are not supported",
-                    ))
-                }
-            }
-        }
-        Fields::Unit => {
-            return Err(Error::new_spanned(
-                node,
-                "unit structs as deserialize are not supported",
-            ));
-        }
-    }
-}
-
-fn derive_c_struct(
+fn derive_struct(
     node: &DeriveInput,
     _strut: &DataStruct,
     named_fields: &FieldsNamed,
@@ -64,63 +66,104 @@ fn derive_c_struct(
     let ty = &node.ident;
     let (impl_generics, ty_generics, where_clause) = node.generics.split_for_impl();
 
-    let fn_body = {
-        let mut members = vec![];
+    enum FieldKind {
+        Ordinary(u32),
+        Optional(u32),
+        Flatten,
+    }
+
+    let fields = {
+        let mut fields = vec![];
+        let mut tags = vec![];
         for field in &named_fields.named {
             let ident = field.ident.clone().unwrap();
-            let tag = common::get_field_tag(&field)?;
-            let opt = common::is_optional(&field)?;
             let ty = field.ty.clone();
-            members.push((ident, tag, opt, ty))
-        }
-
-        let mut init = vec![];
-        for (ident, _tag, opt, ty) in &members {
-            let push = if *opt {
-                quote! {
-                    let mut #ident: #ty = None;
-                }
+            let attrs = attr::get(&field.attrs)?;
+            attrs.disallow_untagged()?;
+            let kind = if attrs.flatten.is_some() {
+                attrs.disallow_tag()?;
+                attrs.disallow_optional()?;
+                FieldKind::Flatten
             } else {
-                quote! {
-                    let mut #ident: ::std::option::Option<#ty> = None;
+                attrs.require_tag(field)?;
+                attr::check_tag_uniqueness(attrs.tag.as_ref().unwrap(), &mut tags)?;
+                let tag = attrs.tag.unwrap().tag;
+                // TODO: require `#[required]` or `#[optional]` for fields of the Option<T> type
+                if attrs.optional.is_some() {
+                    FieldKind::Optional(tag)
+                } else {
+                    FieldKind::Ordinary(tag)
                 }
             };
-            init.push(push);
+            fields.push((ident, ty, kind));
+        }
+        fields
+    };
+
+    let fn_body = {
+        let mut init = vec![];
+        for (ident, ty, kind) in &fields {
+            let code = match kind {
+                FieldKind::Ordinary(_) => {
+                    quote! {
+                        let mut #ident: ::std::option::Option<#ty> = None;
+                    }
+                }
+                FieldKind::Optional(_) => {
+                    quote! {
+                        let mut #ident: #ty = None;
+                    }
+                }
+                FieldKind::Flatten => {
+                    quote! {
+                        let #ident: #ty = __deserializer.clone().deserialize()?;
+                    }
+                }
+            };
+            init.push(code);
         }
 
         let mut filters = vec![];
-        for (ident, tag, _opt, _ty) in &members {
-            filters.push(quote! {
-                #tag => {
-                    if #ident.is_some() {
-                        return Err(::msgpack_schema::InvalidInputError.into());
-                    }
-                    #ident = Some(__deserializer.deserialize()?);
+        for (ident, _, kind) in &fields {
+            match kind {
+                FieldKind::Ordinary(tag) | FieldKind::Optional(tag) => {
+                    filters.push(quote! {
+                        #tag => {
+                            if #ident.is_some() {
+                                return Err(::msgpack_schema::InvalidInputError.into());
+                            }
+                            #ident = Some(__deserializer.deserialize()?);
+                        }
+                    });
                 }
-            });
+                FieldKind::Flatten => {}
+            }
         }
 
         let mut ctors = vec![];
-        for (ident, _tag, opt, _ty) in &members {
-            let push = if *opt {
-                quote! {
-                    #ident,
+        for (ident, _, kind) in &fields {
+            let code = match kind {
+                FieldKind::Ordinary(_) => {
+                    quote! {
+                        #ident: #ident.ok_or(::msgpack_schema::ValidationError)?,
+                    }
                 }
-            } else {
-                quote! {
-                    #ident: #ident.ok_or(::msgpack_schema::ValidationError)?,
+                FieldKind::Optional(_) | FieldKind::Flatten => {
+                    quote! {
+                        #ident,
+                    }
                 }
             };
-            ctors.push(push);
+            ctors.push(code);
         }
 
         quote! {
+            #( #init )*
+
             let __len = match __deserializer.deserialize_token()? {
                 ::msgpack_schema::Token::Map(len) => len,
                 _ => return Err(::msgpack_schema::ValidationError.into()),
             };
-
-            #( #init )*
             for _ in 0..__len {
                 let __tag: u32 = __deserializer.deserialize()?;
                 match __tag {
@@ -151,11 +194,17 @@ fn derive_c_struct(
 fn derive_newtype_struct(
     node: &DeriveInput,
     _strut: &DataStruct,
-    _unnamed: &syn::Field,
+    field: &syn::Field,
 ) -> Result<TokenStream> {
     let ty = &node.ident;
     let (impl_generics, ty_generics, where_clause) = node.generics.split_for_impl();
 
+    let attrs = attr::get(&field.attrs)?;
+    attrs.disallow_tag()?;
+    attrs.disallow_optional()?;
+    attrs.disallow_untagged()?;
+    attrs.disallow_flatten()?;
+
     let fn_body = quote! {
         __deserializer.deserialize().map(Self)
     };
@@ -178,9 +227,16 @@ fn derive_enum(node: &DeriveInput, enu: &DataEnum) -> Result<TokenStream> {
 
     let fn_body = {
         let mut clauses = vec![];
+        let mut tags = vec![];
         for variant in &enu.variants {
             let ident = variant.ident.clone();
-            let tag = common::get_variant_tag(&variant)?;
+            let attrs = attr::get(&variant.attrs)?;
+            attrs.disallow_optional()?;
+            attrs.disallow_untagged()?;
+            attrs.disallow_flatten()?;
+            attrs.require_tag(variant)?;
+            attr::check_tag_uniqueness(attrs.tag.as_ref().unwrap(), &mut tags)?;
+            let tag = attrs.tag.unwrap().tag;
             match &variant.fields {
                 Fields::Named(_) => {
                     return Err(Error::new_spanned(
@@ -202,6 +258,11 @@ fn derive_enum(node: &DeriveInput, enu: &DataEnum) -> Result<TokenStream> {
                             });
                         }
                         1 => {
+                            let attrs = attr::get(&fields.unnamed[0].attrs)?;
+                            attrs.disallow_optional()?;
+                            attrs.disallow_tag()?;
+                            attrs.disallow_untagged()?;
+                            attrs.disallow_flatten()?;
                             clauses.push(quote! {
                                 #tag => {
                                     if !__is_array {
@@ -270,6 +331,11 @@ fn derive_untagged_enum(node: &DeriveInput, enu: &DataEnum) -> Result<TokenStrea
     let fn_body = {
         let mut members = vec![];
         for variant in &enu.variants {
+            let attrs = attr::get(&variant.attrs)?;
+            attrs.disallow_optional()?;
+            attrs.disallow_tag()?;
+            attrs.disallow_untagged()?;
+            attrs.disallow_flatten()?;
             match &variant.fields {
                 Fields::Named(_) => {
                     return Err(Error::new_spanned(
@@ -285,6 +351,11 @@ fn derive_untagged_enum(node: &DeriveInput, enu: &DataEnum) -> Result<TokenStrea
                         ));
                     }
                     1 => {
+                        let attrs = attr::get(&fields.unnamed[0].attrs)?;
+                        attrs.disallow_optional()?;
+                        attrs.disallow_tag()?;
+                        attrs.disallow_untagged()?;
+                        attrs.disallow_flatten()?;
                         members.push((variant, &fields.unnamed[0]));
                     }
                     _ => {
@@ -332,7 +403,7 @@ fn derive_untagged_enum(node: &DeriveInput, enu: &DataEnum) -> Result<TokenStrea
     Ok(gen)
 }
 
-fn derive_untagged_c_struct(
+fn derive_untagged_struct(
     node: &DeriveInput,
     _strut: &DataStruct,
     named_fields: &FieldsNamed,
@@ -343,6 +414,11 @@ fn derive_untagged_c_struct(
     let fn_body = {
         let mut members = vec![];
         for field in &named_fields.named {
+            let attrs = attr::get(&field.attrs)?;
+            attrs.disallow_tag()?;
+            attrs.disallow_optional()?;
+            attrs.disallow_untagged()?;
+            attrs.disallow_flatten()?;
             let ident = field.ident.clone().unwrap();
             let ty = field.ty.clone();
             members.push((ident, ty))
